@@ -17,9 +17,11 @@
 """Juniper JCL generator."""
 
 
+from typing import Dict, List, Set, Tuple, Union
+
 from absl import logging
 
-from aerleon.lib import aclgenerator, nacaddr, summarizer
+from aerleon.lib import aclgenerator, nacaddr, policy, summarizer
 
 
 # generic error class
@@ -75,7 +77,7 @@ class Config:
       lines: the text lines of the configuration.
     """
 
-    def __init__(self, indent=0, tabstop=4):
+    def __init__(self, indent: int = 0, tabstop: int = 4):
         self.indent = indent
         self._initial_indent = indent
         self.tabstop = tabstop
@@ -88,7 +90,7 @@ class Config:
             )
         return '\n'.join(self.lines)
 
-    def Append(self, line, verbatim=False):
+    def Append(self, line: str, verbatim=False):
         """Append one line to the configuration.
 
         Args:
@@ -179,10 +181,24 @@ class Term(aclgenerator.Term):
             'protocol-except': 'ip-protocol-except',
             'tcp-est': 'tcp-flags "(ack|rst)"',
         },
+        'ethernet-switching': {
+            'addr': 'ip-address',
+            'saddr': 'ip-source-address',
+            'daddr': 'ip-destination-address',
+            'protocol': 'ip-protocol',
+            'protocol-except': 'ip-protocol-except',
+            'tcp-est': 'tcp-established',
+        },
     }
 
     def __init__(
-        self, term, term_type, enable_dsmo, noverbose, filter_direction=None, interface_type=None
+        self,
+        term: policy.Term,
+        term_type: str,
+        enable_dsmo: bool,
+        noverbose: bool,
+        filter_direction: str = None,
+        interface_type: str = None,
     ):
         super().__init__(term)
         self.term = term
@@ -206,9 +222,6 @@ class Term(aclgenerator.Term):
             # some options need to modify the actions
             self.extra_actions = []
 
-    # TODO(pmoody): get rid of all of the default string concatenation here.
-    #  eg, indent(8) + 'foo;' -> '%s%s;' % (indent(8), 'foo'). pyglint likes this
-    #  more.
     def __str__(self):
         config = Config(indent=self._DEFAULT_INDENT)
         from_str = []
@@ -283,6 +296,8 @@ class Term(aclgenerator.Term):
                     from_str.append('tcp-initial;')
                 elif opt.startswith('first-fragment'):
                     from_str.append('first-fragment;')
+                elif opt.startswith('not-syn-ack'):
+                    from_str.append('tcp-flags "!(syn&ack)";')
 
                 # we don't have a special way of dealing with this, so we output it and
                 # hope the user knows what they're doing.
@@ -701,7 +716,7 @@ class Term(aclgenerator.Term):
         return str(config)
 
     @staticmethod
-    def NextIpCheck(next_ip, term_name):
+    def NextIpCheck(next_ip: List[Union[nacaddr.IPv4, nacaddr.IPv6]], term_name: str):
         if len(next_ip) > 1:
             raise JuniperNextIpError(
                 'The following term has more ' 'than one next IP value: %s' % term_name
@@ -722,7 +737,11 @@ class Term(aclgenerator.Term):
                 'The following term has multiple terminating actions: %s' % self.term.name
             )
 
-    def _MinimizePrefixes(self, include, exclude):
+    def _MinimizePrefixes(
+        self,
+        include: List[Union[nacaddr.IPv4, nacaddr.IPv6]],
+        exclude: List[Union[nacaddr.IPv4, nacaddr.IPv6]],
+    ):
         """Calculate a minimal set of prefixes for Juniper match conditions.
 
         Args:
@@ -756,7 +775,12 @@ class Term(aclgenerator.Term):
 
         return include_result, exclude_result
 
-    def _Comment(self, addr, exclude=False, line_length=132):
+    def _Comment(
+        self,
+        addr: Union[nacaddr.IPv4, nacaddr.IPv6, summarizer.DSMNet],
+        exclude: bool = False,
+        line_length: int = 132,
+    ):
         """Returns address comment field if it exists.
 
         Args:
@@ -848,7 +872,7 @@ class Term(aclgenerator.Term):
             logging.warning('Ignoring non IPv4 or IPv6 address: %s', addr)
         return rval
 
-    def _Group(self, group, lc=True):
+    def _Group(self, group: List[str], lc: bool = True):
         """If 1 item return it, else return [ item1 item2 ].
 
         Args:
@@ -861,7 +885,7 @@ class Term(aclgenerator.Term):
                 or with just ';' appended if len(group) == 1
         """
 
-        def _FormattedGroup(el, lc=True):
+        def _FormattedGroup(el: List[str], lc: bool = True):
             """Return the actual formatting of an individual element.
 
             Args:
@@ -872,7 +896,7 @@ class Term(aclgenerator.Term):
               string: either the lower()'ed string or the ports, hyphenated
                       if they're a range, or by itself if it's not.
             """
-            if isinstance(el, str) or isinstance(el, str):
+            if isinstance(el, str):
                 if lc:
                     return el
                 else:
@@ -904,11 +928,11 @@ class Juniper(aclgenerator.ACLGenerator):
 
     _PLATFORM = 'juniper'
     _DEFAULT_PROTOCOL = 'ip'
-    _SUPPORTED_AF = frozenset(('inet', 'inet6', 'bridge', 'mixed'))
+    _SUPPORTED_AF = frozenset(('inet', 'inet6', 'bridge', 'ethernet-switching', 'mixed'))
     _TERM = Term
     SUFFIX = '.jcl'
 
-    def _BuildTokens(self):
+    def _BuildTokens(self) -> Tuple[Set[str], Dict[str, Set[str]]]:
         """Build supported tokens for platform.
 
         Returns:
@@ -964,12 +988,13 @@ class Juniper(aclgenerator.ACLGenerator):
                     'tcp-established',
                     'tcp-initial',
                     'inactive',
+                    'not-syn-ack',
                 }
             }
         )
         return supported_tokens, supported_sub_tokens
 
-    def _TranslatePolicy(self, pol, exp_info):
+    def _TranslatePolicy(self, pol: policy.Policy, exp_info: int):
         self.juniper_policies = []
         for header, terms in pol.filters:
             filter_options = header.FilterOptions(self._PLATFORM)
