@@ -56,6 +56,7 @@ from yaml import YAMLError
 
 from aerleon.lib import nacaddr
 from aerleon.lib import port as portlib
+from aerleon.lib.fqdn import FQDN
 from aerleon.lib.nacaddr import IPv4, IPv6
 from aerleon.lib.yaml_loader import SpanSafeYamlLoader
 
@@ -72,7 +73,7 @@ class NamespaceCollisionError(Error):
 
 
 class BadNetmaskTypeError(Error):
-    """Used to report on duplicate symbol names found while parsing."""
+    """Deprecated."""
 
 
 class NoDefinitionsError(Error):
@@ -107,6 +108,10 @@ class DefinitionFileTypeError(Error):
     """Invalid Definition File"""
 
 
+class EmptyDefinitionError(Error):
+    """A token returned no results."""
+
+
 # Consider making this span-oriented
 # (file > line > (start_ch, end_ch))
 class UserMessage:
@@ -125,18 +130,20 @@ class UserMessage:
             The top-level file should be the first item in the list.
     """
 
-    message: str
-    filename: str
-    line: int
-    include_chain: "list[Tuple[str, int]]"
-
-    def __init__(self, message, *, filename, line=None, include_chain=None):
+    def __init__(
+        self,
+        message: str,
+        *,
+        filename: str,
+        line: int = None,
+        include_chain: List[Tuple[str, int]] = None,
+    ):
         self.message = message
         self.filename = filename
         self.line = line
         self.include_chain = include_chain
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Display user-facing error message with include chain (if present).
 
         e.g.
@@ -161,11 +168,11 @@ class UserMessage:
                     error_context += " (Top Level)"
         return error_context
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"UserMessage(\"{str(self)}\")"
 
 
-def is_yaml_suffix(suffix):
+def is_yaml_suffix(suffix: str) -> bool:
     return suffix == '.yaml' or suffix == '.yml'
 
 
@@ -214,6 +221,7 @@ class Naming:
         self.current_symbol = None
         self.services = {}
         self.networks = {}
+        self.fqdn = {}
         self.unseen_services = {}
         self.unseen_networks = {}
         self.port_re = re.compile(r'(^\d+-\d+|^\d+)\/\w+$|^[\w\d-]+$', re.IGNORECASE | re.DOTALL)
@@ -411,7 +419,7 @@ class Naming:
         except ValueError:
             return False
 
-    def GetServiceNames(self):
+    def GetServiceNames(self) -> List[str]:
         """Returns the list of all known service names."""
         return list(self.services.keys())
 
@@ -455,7 +463,7 @@ class Naming:
                 expandset.add(service)
         return sorted(expandset)
 
-    def GetPortParents(self, query, proto):
+    def GetPortParents(self, query: str, proto: str) -> List[str]:
         """Returns a list of all service tokens containing the port/protocol pair.
 
         Args:
@@ -536,43 +544,80 @@ class Naming:
                     services_set.add(parts[0])
         return sorted(services_set)
 
-    def GetNetAddr(self, token: str) -> List[Union[IPv4, IPv6]]:
-        """Given a network token, return a list of nacaddr.IPv4 or nacaddr.IPv6 objects.
+    def GetFQDN(self, query: str) -> List[str]:
+        """Expand a network token into a list of FQDN objects.
 
         Args:
-          token: A name of a network definition, such as 'INTERNAL'
-
-        Returns:
-          A list of nacaddr.IPv4 or nacaddr.IPv6 objects.
-
-        Raises:
-          UndefinedAddressError: if the network name isn't defined.
-        """
-        return self.GetNet(token)
-
-    def GetNet(self, query: str) -> List[Union[IPv4, IPv6]]:
-        """Expand a network token into a list of nacaddr.IPv4 or nacaddr.IPv6 objects.
-
-        Args:
-          query: Network definition token which may include comment text
-
-        Raises:
-          BadNetmaskTypeError: Results when an unknown netmask_type is
-          specified.  Acceptable values are 'cidr', 'netmask', and 'hostmask'.
+          query: Network definition token. May include comment text
 
         Returns:
           List of nacaddr.IPv4 or nacaddr.IPv6 objects
 
         Raises:
-          UndefinedAddressError: for an undefined token value
+          UndefinedAddressError: Network token not defined
+          EmptyDefinitionError: No FQDN values found for this network token
         """
+        results = self._GetFQDN(query)
+        if len(results) == 0:
+            raise EmptyDefinitionError(f"No FQDN values found for network: {query}")
+        return results
+
+    def _GetFQDN(self, query: str, level=0) -> List[str]:
         returnlist = []
-        data = []
-        token = ''
-        data = query.split('#')  # Get the token keyword and remove any comment
-        token = data[0].split()[0]  # Remove whitespace and cast from list to string
+        data = query.split('#')
+        token = data[0].split()[0]
+        if token not in self.fqdn:
+            raise UndefinedAddressError(f'UNDEFINED: {token}')
+        for i in self.fqdn[token].items:
+            comment = ''
+            if i.find('#') > -1:
+                (name, comment) = i.split('#', 1)
+            else:
+                name = i
+
+            name = name.strip()
+            if self.token_re.match(name):
+                returnlist.extend(self._GetFQDN(name))
+            else:
+                try:
+                    fqdn = FQDN(name, token, comment)
+                    fqdn.text = comment.lstrip()
+                    fqdn.token = token
+                    returnlist.append(fqdn)
+                except ValueError:
+                    pass
+        for i in returnlist:
+            i.parent_token = token
+        return returnlist
+
+    def GetNetAddr(self, query: str) -> List[Union[IPv4, IPv6]]:
+        """Alias of Naming.GetNet"""
+        return self.GetNet(query)
+
+    def GetNet(self, query: str) -> List[Union[IPv4, IPv6]]:
+        """Expand a network token into a list of nacaddr.IPv4 or nacaddr.IPv6 objects.
+
+        Args:
+          query: Network definition token. May include comment text.
+
+        Returns:
+          List of nacaddr.IPv4 or nacaddr.IPv6 objects
+
+        Raises:
+          UndefinedAddressError: Network token not defined
+          EmptyDefinitionError: No IP address values found for this network token
+        """
+        results = self._GetNet(query)
+        if len(results) == 0:
+            raise EmptyDefinitionError(f"No IP addresses found for network: {query}")
+        return results
+
+    def _GetNet(self, query: str) -> List[Union[IPv4, IPv6]]:
+        returnlist = []
+        data = query.split('#')
+        token = data[0].split()[0]
         if token not in self.networks:
-            raise UndefinedAddressError('%s %s' % ('\nUNDEFINED:', str(token)))
+            raise UndefinedAddressError(f'UNDEFINED: {token}')
 
         for i in self.networks[token].items:
             comment = ''
@@ -586,20 +631,17 @@ class Naming:
                 returnlist.extend(self.GetNet(net))
             else:
                 try:
-                    # TODO(robankeny): Fix using error to continue processing.
                     addr = nacaddr.IP(net, strict=False)
                     addr.text = comment.lstrip()
                     addr.token = token
                     returnlist.append(addr)
                 except ValueError:
-                    # if net was something like 'FOO', or the name of another token which
-                    # needs to be dereferenced, nacaddr.IP() will return a ValueError
-                    returnlist.extend(self.GetNet(net))
+                    pass
         for i in returnlist:
             i.parent_token = token
         return returnlist
 
-    def _Parse(self, definitions_directory):
+    def _Parse(self, definitions_directory: str) -> None:
         """Parse files for tokens and values.
 
         Given a directory name, grab all the appropriate files in that
@@ -612,7 +654,6 @@ class Naming:
         Raises:
           NoDefinitionsError: if no definitions are found.
         """
-
         file_def_type = {
             '.net': DEF_TYPE_NETWORKS,
             '.svc': DEF_TYPE_SERVICES,
@@ -684,13 +725,6 @@ class Naming:
           NamingSyntaxError: Syntax error parsing config.
         """
 
-        #
-        # NOTE: The "unseen name" logic defined in this function (_ParseLine) is duplicated in
-        # function ParseDefinitionsObject. Any changes to how "unseen name" checking is done
-        # need to be made in both places.
-        #
-        # TODO(jb): Consider splitting up _ParseLine so that it generates an intermediate
-        #  representation (ItemUnit) and "unseen name" checks are done on the IR (by both _Parse* flows).
         if definition_type not in ['services', 'networks']:
             raise UnexpectedDefinitionTypeError(
                 '%s %s' % ('Received an unexpected definition type:', definition_type)
@@ -769,16 +803,16 @@ class Naming:
                             if value_piece not in self.unseen_networks:
                                 self.unseen_networks[value_piece] = True
 
-    def ParseYaml(self, file_handle: str, file_name: str) -> None:
-        """Load a definition yaml file as a string.
+    def ParseYaml(self, file: str, file_name: str) -> None:
+        """Load network and service definitions from YAML.
 
         Arguments:
             file: A string containing the file contents.
-            filename: The original filename of the file.
+            file_name: The original filename of the file.
         """
 
         try:
-            file_data = yaml.load(file_handle, Loader=SpanSafeYamlLoader(filename=file_name))
+            file_data = yaml.load(file, Loader=SpanSafeYamlLoader(filename=file_name))
         except YAMLError as yaml_error:
             raise DefinitionFileTypeError(
                 UserMessage("Unable to read file as YAML.", filename=file_name)
@@ -787,6 +821,13 @@ class Naming:
         self.ParseDefinitionsObject(file_data, file_name)
 
     def ParseDefinitionsObject(self, file_data: Dict[str, str], file_name: str) -> None:
+        """Load network and service definitions from a Python object.
+
+        Arguments:
+            file_data: A Python dict where file_data.networks contains network
+                data and/or file_data.services contains service data.
+            file_name: The original filename of the file.
+        """
         # Empty files are ignored with a warning
         if not file_data:
             logging.warning(UserMessage("Ignoring empty address book file.", filename=file_name))
@@ -827,22 +868,21 @@ class Naming:
                 )
                 continue
 
-            # TODO(jb) This check should be performed on the IR so we can hoist it from _ParseLine
             if not self.token_re.match(symbol):
                 logging.info(
                     f'\nNetwork name does not match recommended criteria: {symbol}\nOnly A-Z, a-z, 0-9, -, and _ allowed'
                 )
 
-            # TODO(jb) This check should be performed on the IR so we can hoist it from _ParseLine
             if symbol in self.networks:
                 raise NamespaceCollisionError(
                     f'\nMultiple definitions found for network: {symbol}'
                 )
 
-            unit = _ItemUnit(symbol)
+            addr_unit = _ItemUnit(symbol)
+            fqdn_unit = _ItemUnit(symbol)
 
-            # TODO(jb) This operation should be performed on the IR so we can hoist it from _ParseLine
-            self.networks[symbol] = unit
+            self.networks[symbol] = addr_unit
+            self.fqdn[symbol] = fqdn_unit
             if symbol in self.unseen_networks:
                 self.unseen_networks.pop(symbol)
 
@@ -851,6 +891,7 @@ class Naming:
                 # 1. A string, understood as a network name reference
                 # 2. A dictionary, with these fields:
                 #    'address': A specific IP address or CIDR range
+                #    'fqdn': A FQDN for use in DNS filtering.
                 #    'name': A network name reference
                 #    'comment': An optional comment
                 # 'address' or 'name' must be present in any dictionary item
@@ -865,6 +906,8 @@ class Naming:
                         value = network_ref = item['name']
                     elif 'address' in item and isinstance(item['address'], str):
                         value = ip = item['address']
+                    elif 'fqdn' in item and isinstance(item['fqdn'], str):
+                        value = item['fqdn']
                     else:
                         logging.info(f'\nNetwork name or CIDR expected for: {symbol}')
                         continue
@@ -874,11 +917,22 @@ class Naming:
                 else:
                     logging.info(f'\nUnexpected symbol definition: {symbol}')
                     continue
-
                 if comment is None:
-                    unit.items.append(value)
+                    if network_ref:
+                        addr_unit.items.append(value)
+                        fqdn_unit.items.append(value)
+                    elif ip:
+                        addr_unit.items.append(value)
+                    else:
+                        fqdn_unit.items.append(value)
                 else:
-                    unit.items.append(f'{value} # {comment}')
+                    if network_ref:
+                        addr_unit.items.append(f'{value} # {comment}')
+                        fqdn_unit.items.append(f'{value} # {comment}')
+                    elif ip:
+                        addr_unit.items.append(f'{value} # {comment}')
+                    else:
+                        fqdn_unit.items.append(f'{value} # {comment}')
 
                 if network_ref and network_ref not in self.networks:
                     if network_ref not in self.unseen_networks:
@@ -904,13 +958,11 @@ class Naming:
                 )
                 continue
 
-            # TODO(jb) This check should be performed on the IR so we can hoist it from _ParseLine
             if not self.token_re.match(symbol):
                 logging.info(
                     f'\nService name does not match recommended criteria: {symbol}\nOnly A-Z, a-z, 0-9, -, and _ allowed'
                 )
 
-            # TODO(jb) This check should be performed on the IR so we can hoist it from _ParseLine
             if symbol in self.services:
                 raise NamespaceCollisionError(
                     f'\nMultiple definitions found for service: {symbol}'
@@ -918,7 +970,6 @@ class Naming:
 
             unit = _ItemUnit(symbol)
 
-            # TODO(jb) This operation should be performed on the IR so we can hoist it from _ParseLine
             self.services[symbol] = unit
             if symbol in self.unseen_services:
                 self.unseen_services.pop(symbol)
